class ArchiveError(Exception): ...
class Bad7zFile(ArchiveError): ...
class CrcError(ArchiveError): ...
class UnsupportedCompressionMethodError(ArchiveError): ...
class DecompressionError(ArchiveError): ...
class InternalError(ArchiveError): ...
class PasswordRequired(Exception): ...



import argparse
from py7zr.callbacks import ExtractCallback as ExtractCallback
from py7zr.compressor import SupportedMethods as SupportedMethods
from py7zr.helpers import Local as Local
from py7zr.properties import COMMAND_HELP_STRING as COMMAND_HELP_STRING
from typing import Any, Optional

class CliExtractCallback(ExtractCallback):
    ofd: Any = ...
    archive_total: Any = ...
    total_bytes: int = ...
    pwidth: int = ...
    def __init__(self, total_bytes: Any, ofd: Any = ...) -> None: ...
    def report_start_preparation(self) -> None: ...
    def report_start(self, processing_file_path: Any, processing_bytes: Any) -> None: ...
    def report_end(self, processing_file_path: Any, wrote_bytes: Any) -> None: ...
    def report_postprocess(self) -> None: ...
    def report_warning(self, message: Any) -> None: ...

class Cli:
    dunits: Any = ...
    parser: Any = ...
    unit_pattern: Any = ...
    def __init__(self) -> None: ...
    def run(self, arg: Optional[Any]=...) -> int: ...
    def show_version(self) -> None: ...
    def show_help(self, args: Any): ...
    def run_info(self, args: Any) -> None: ...
    def run_list(self, args: Any): ...
    @staticmethod
    def print_archiveinfo(archive: Any, file: Any) -> None: ...
    def run_test(self, args: Any): ...
    def run_extract(self, args: argparse.Namespace) -> int: ...
    def run_create(self, args: Any): ...
    def run_append(self, args: Any): ...



import pathlib
from datetime import tzinfo
from typing import Any, BinaryIO, Optional, Union

def calculate_crc32(data: bytes, value: int=..., blocksize: int=...) -> int: ...

calculate_key: Any

def filetime_to_dt(ft: Any): ...

ZERO: Any
HOUR: Any
SECOND: Any
STDOFFSET: Any
DSTOFFSET: Any
DSTOFFSET = STDOFFSET
DSTDIFF: Any

class LocalTimezone(tzinfo):
    def fromutc(self, dt: Any): ...
    def utcoffset(self, dt: Any): ...
    def dst(self, dt: Any): ...
    def tzname(self, dt: Any): ...

Local: Any
TIMESTAMP_ADJUST: int

class UTC(tzinfo):
    def utcoffset(self, dt: Any): ...
    def tzname(self, dt: Any): ...
    def dst(self, dt: Any): ...
    def _call__(self): ...

class ArchiveTimestamp(int):
    def __index__(self): ...
    def totimestamp(self) -> float: ...
    def as_datetime(self): ...
    @staticmethod
    def from_datetime(val: Any): ...
    @staticmethod
    def from_now(): ...

def islink(path: Any): ...
def readlink(path: Union[str, pathlib.Path], *, dir_fd: Any=...) -> Union[str, pathlib.Path]: ...

class MemIO:
    def __init__(self, buf: BinaryIO) -> None: ...
    def write(self, data: bytes) -> int: ...
    def read(self, length: Optional[int]=...) -> bytes: ...
    def close(self) -> None: ...
    def flush(self) -> None: ...
    def seek(self, position: int) -> None: ...
    def open(self, mode: Optional[Any] = ...): ...
    @property
    def parent(self): ...
    def mkdir(self, parents: Optional[Any] = ..., exist_ok: bool = ...) -> None: ...
    def __enter__(self): ...
    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None: ...

class NullIO:
    def __init__(self) -> None: ...
    def write(self, data: Any): ...
    def read(self, length: Optional[Any] = ...): ...
    def close(self) -> None: ...
    def flush(self) -> None: ...
    def open(self, mode: Optional[Any] = ...): ...
    @property
    def parent(self): ...
    def mkdir(self) -> None: ...
    def __enter__(self): ...
    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None: ...

class BufferOverflow(Exception): ...

class Buffer:
    view: Any = ...
    def __init__(self, size: int=...) -> None: ...
    def add(self, data: Union[bytes, bytearray, memoryview]) -> Any: ...
    def reset(self) -> None: ...
    def set(self, data: Union[bytes, bytearray, memoryview]) -> None: ...
    def get(self) -> bytearray: ...
    def __len__(self) -> int: ...
    def __bytes__(self): ...



from typing import Any

MAGIC_7Z: Any
FINISH_7Z: Any
COMMAND_HELP_STRING: str

def get_default_blocksize(): ...

FILTER_LZMA: Any
FILTER_LZMA2: Any
FILTER_DELTA: Any
FILTER_ARM: Any
FILTER_ARMTHUMB: Any
FILTER_IA64: Any
FILTER_POWERPC: Any
FILTER_SPARC: Any
FILTER_X86: Any
CHECK_CRC32: Any
CHECK_CRC64: Any
CHECK_SHA256: Any
CHECK_NONE: Any
CHECK_ID_MAX: Any
CHECK_UNKNOWN: Any
PRESET_DEFAULT: Any
PRESET_EXTREME: Any
FILTER_CRYPTO_AES256_SHA256: int
FILTER_CRYPTO_ZIP: int
FILTER_CRYPTO_RAR29: int
FILTER_BZIP2: int
FILTER_DEFLATE: int
FILTER_COPY: int
FILTER_ZSTD: int
FILTER_PPMD: int
FILTER_BROTLI: int

class Constant:
    def __setattr__(self, *_: Any) -> None: ...

class DefaultFilters(Constant):
    ARCHIVE_FILTER: Any = ...
    ENCODED_HEADER_FILTER: Any = ...
    ENCRYPTED_ARCHIVE_FILTER: Any = ...
    ENCRYPTED_HEADER_FILTER: Any = ...

DEFAULT_FILTERS: Any

class Property(Constant):
    END: Any = ...
    HEADER: Any = ...
    ARCHIVE_PROPERTIES: Any = ...
    ADDITIONAL_STREAMS_INFO: Any = ...
    MAIN_STREAMS_INFO: Any = ...
    FILES_INFO: Any = ...
    PACK_INFO: Any = ...
    UNPACK_INFO: Any = ...
    SUBSTREAMS_INFO: Any = ...
    SIZE: Any = ...
    CRC: Any = ...
    FOLDER: Any = ...
    CODERS_UNPACK_SIZE: Any = ...
    NUM_UNPACK_STREAM: Any = ...
    EMPTY_STREAM: Any = ...
    EMPTY_FILE: Any = ...
    ANTI: Any = ...
    NAME: Any = ...
    CREATION_TIME: Any = ...
    LAST_ACCESS_TIME: Any = ...
    LAST_WRITE_TIME: Any = ...
    ATTRIBUTES: Any = ...
    COMMENT: Any = ...
    ENCODED_HEADER: Any = ...
    START_POS: Any = ...
    DUMMY: Any = ...

PROPERTY: Any

class CompressionMethod(Constant):
    COPY: Any = ...
    DELTA: Any = ...
    BCJ: Any = ...
    PPC: Any = ...
    IA64: Any = ...
    ARM: Any = ...
    ARMT: Any = ...
    SPARC: Any = ...
    SWAP2: Any = ...
    SWAP4: Any = ...
    LZMA: Any = ...
    PPMD: Any = ...
    P7Z_BCJ: Any = ...
    P7Z_BCJ2: Any = ...
    BCJ_PPC: Any = ...
    BCJ_IA64: Any = ...
    BCJ_ARM: Any = ...
    BCJ_ARMT: Any = ...
    BCJ_SPARC: Any = ...
    LZMA2: Any = ...
    MISC_ZIP: Any = ...
    MISC_BZIP2: Any = ...
    MISC_DEFLATE: Any = ...
    MISC_DEFLATE64: Any = ...
    MISC_Z: Any = ...
    MISC_LZH: Any = ...
    NSIS_DEFLATE: Any = ...
    NSIS_BZIP2: Any = ...
    MISC_ZSTD: Any = ...
    MISC_BROTLI: Any = ...
    MISC_LZ4: Any = ...
    MISC_LZS: Any = ...
    MISC_LIZARD: Any = ...
    CRYPT_ZIPCRYPT: Any = ...
    CRYPT_RAR29AES: Any = ...
    CRYPT_AES256_SHA256: Any = ...

COMPRESSION_METHOD: Any



import abc
from abc import ABC, abstractmethod
from enum import Enum
from py7zr.exceptions import PasswordRequired as PasswordRequired, UnsupportedCompressionMethodError as UnsupportedCompressionMethodError
from py7zr.helpers import Buffer as Buffer, calculate_crc32 as calculate_crc32, calculate_key as calculate_key
from py7zr.properties import COMPRESSION_METHOD as COMPRESSION_METHOD, CompressionMethod as CompressionMethod, FILTER_ARM as FILTER_ARM, FILTER_ARMTHUMB as FILTER_ARMTHUMB, FILTER_BROTLI as FILTER_BROTLI, FILTER_BZIP2 as FILTER_BZIP2, FILTER_COPY as FILTER_COPY, FILTER_CRYPTO_AES256_SHA256 as FILTER_CRYPTO_AES256_SHA256, FILTER_DEFLATE as FILTER_DEFLATE, FILTER_DELTA as FILTER_DELTA, FILTER_IA64 as FILTER_IA64, FILTER_LZMA as FILTER_LZMA, FILTER_LZMA2 as FILTER_LZMA2, FILTER_POWERPC as FILTER_POWERPC, FILTER_PPMD as FILTER_PPMD, FILTER_SPARC as FILTER_SPARC, FILTER_X86 as FILTER_X86, FILTER_ZSTD as FILTER_ZSTD, MAGIC_7Z as MAGIC_7Z, get_default_blocksize as get_default_blocksize
from typing import Any, Dict, List, Optional, Tuple, Union

brotli_major: int
brotli_minor: int

class ISevenZipCompressor(ABC, metaclass=abc.ABCMeta):
    @abstractmethod
    def compress(self, data: Union[bytes, bytearray, memoryview]) -> bytes: ...
    @abstractmethod
    def flush(self) -> bytes: ...

class ISevenZipDecompressor(ABC, metaclass=abc.ABCMeta):
    @abstractmethod
    def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int=...) -> bytes: ...

class AESCompressor(ISevenZipCompressor):
    AES_CBC_BLOCKSIZE: int = ...
    cycles: int = ...
    iv: Any = ...
    salt: bytes = ...
    method: Any = ...
    cipher: Any = ...
    flushed: bool = ...
    buf: Any = ...
    def __init__(self, password: str, blocksize: Optional[int]=...) -> None: ...
    def encode_filter_properties(self): ...
    def compress(self, data: Any): ...
    def flush(self): ...

class AESDecompressor(ISevenZipDecompressor):
    cipher: Any = ...
    buf: Any = ...
    def __init__(self, aes_properties: bytes, password: str, blocksize: Optional[int]=...) -> None: ...
    def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int=...) -> bytes: ...

class DeflateCompressor(ISevenZipCompressor):
    def __init__(self) -> None: ...
    def compress(self, data: Any): ...
    def flush(self): ...

class DeflateDecompressor(ISevenZipDecompressor):
    flushed: bool = ...
    def __init__(self) -> None: ...
    def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int=...) -> bytes: ...

class CopyCompressor(ISevenZipCompressor):
    def compress(self, data: Union[bytes, bytearray, memoryview]) -> bytes: ...
    def flush(self): ...

class CopyDecompressor(ISevenZipDecompressor):
    def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int=...) -> bytes: ...

class PpmdDecompressor(ISevenZipDecompressor):
    decoder: Any = ...
    def __init__(self, properties: bytes, blocksize: Optional[int]=...) -> None: ...
    def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: Any=...) -> bytes: ...

class PpmdCompressor(ISevenZipCompressor):
    encoder: Any = ...
    def __init__(self, properties: bytes) -> None: ...
    def compress(self, data: Union[bytes, bytearray, memoryview]) -> bytes: ...
    def flush(self): ...
    @classmethod
    def encode_filter_properties(cls, filter: Dict[str, Union[str, int]]) -> Any: ...

class BcjSparcDecoder(ISevenZipDecompressor):
    decoder: Any = ...
    def __init__(self, size: int) -> None: ...
    def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int=...) -> bytes: ...

class BcjSparcEncoder(ISevenZipCompressor):
    encoder: Any = ...
    def __init__(self) -> None: ...
    def compress(self, data: Union[bytes, bytearray, memoryview]) -> bytes: ...
    def flush(self): ...

class BcjPpcDecoder(ISevenZipDecompressor):
    decoder: Any = ...
    def __init__(self, size: int) -> None: ...
    def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int=...) -> bytes: ...

class BcjPpcEncoder(ISevenZipCompressor):
    encoder: Any = ...
    def __init__(self) -> None: ...
    def compress(self, data: Union[bytes, bytearray, memoryview]) -> bytes: ...
    def flush(self): ...

class BcjArmtDecoder(ISevenZipDecompressor):
    decoder: Any = ...
    def __init__(self, size: int) -> None: ...
    def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int=...) -> bytes: ...

class BcjArmtEncoder(ISevenZipCompressor):
    encoder: Any = ...
    def __init__(self) -> None: ...
    def compress(self, data: Union[bytes, bytearray, memoryview]) -> bytes: ...
    def flush(self): ...

class BcjArmDecoder(ISevenZipDecompressor):
    decoder: Any = ...
    def __init__(self, size: int) -> None: ...
    def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int=...) -> bytes: ...

class BcjArmEncoder(ISevenZipCompressor):
    encoder: Any = ...
    def __init__(self) -> None: ...
    def compress(self, data: Union[bytes, bytearray, memoryview]) -> bytes: ...
    def flush(self): ...

class BCJDecoder(ISevenZipDecompressor):
    decoder: Any = ...
    def __init__(self, size: int) -> None: ...
    def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int=...) -> bytes: ...

class BCJEncoder(ISevenZipCompressor):
    encoder: Any = ...
    def __init__(self) -> None: ...
    def compress(self, data: Union[bytes, bytearray, memoryview]) -> bytes: ...
    def flush(self): ...

class BrotliCompressor:
    def __init__(self, level: Any) -> None: ...
    def compress(self, data: Union[bytes, bytearray, memoryview]) -> bytes: ...
    def flush(self) -> bytes: ...

class BrotliDecompressor:
    decompress: Any = ...
    def __init__(self, properties: bytes, block_size: int) -> None: ...

class ZstdCompressor:
    compressor: Any = ...
    def __init__(self, level: int) -> None: ...
    def compress(self, data: Union[bytes, bytearray, memoryview]) -> bytes: ...
    def flush(self) -> bytes: ...

class ZstdDecompressor:
    decompressor: Any = ...
    def __init__(self, properties: bytes, blocksize: int) -> None: ...
    def decompress(self, data: Union[bytes, bytearray, memoryview]) -> bytes: ...

algorithm_class_map: Dict[int, Tuple[Any, Any]]

class LZMA1Decompressor(ISevenZipDecompressor):
    unpacksize: Any = ...
    def __init__(self, filters: Any, unpacksize: Any) -> None: ...
    def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int = ...) -> bytes: ...

class SevenZipDecompressor:
    input_size: Any = ...
    unpacksizes: Any = ...
    consumed: int = ...
    crc: Any = ...
    digest: int = ...
    block_size: Any = ...
    methods_map: Any = ...
    chain: Any = ...
    def __init__(self, coders: List[Dict[str, Any]], packsize: int, unpacksizes: List[int], crc: Optional[int], password: Optional[str]=..., blocksize: Optional[int]=...) -> None: ...
    def decompress(self, fp: Any, max_length: int=...) -> bytes: ...
    def check_crc(self): ...
    @property
    def unused_size(self): ...

class SevenZipCompressor:
    filters: Any = ...
    chain: Any = ...
    digest: int = ...
    packsize: int = ...
    methods_map: Any = ...
    coders: Any = ...
    def __init__(self, filters: Any=..., password: Any=..., blocksize: Optional[int]=...) -> None: ...
    def compress(self, fd: Any, fp: Any, crc: int = ...): ...
    def flush(self, fp: Any): ...
    @property
    def unpacksizes(self): ...

class MethodsType(Enum):
    compressor: int = ...
    filter: int = ...
    crypto: int = ...

class SupportedMethods:
    formats: Any = ...
    methods: Any = ...
    @classmethod
    def get_method_name_id(cls, filter_id: Any): ...
    @classmethod
    def get_filter_id(cls, coder: Any): ...
    @classmethod
    def is_native_filter(cls, filter: Any) -> bool: ...
    @classmethod
    def is_compressor(cls, filter: Any): ...
    @classmethod
    def is_compressor_id(cls, filter_id: Any): ...
    @classmethod
    def is_native_coder(cls, coder: Any) -> bool: ...
    @classmethod
    def need_property(cls, filter_id: Any): ...
    @classmethod
    def is_crypto_id(cls, filter_id: Any) -> bool: ...
    @classmethod
    def get_method_id(cls, filter_id: Any) -> bytes: ...
    @classmethod
    def get_coder(cls, filter: Any) -> Dict[str, Any]: ...
    @classmethod
    def needs_password(cls, coders: Any) -> bool: ...

def get_methods_names_string(coders_lists: List[List[dict]]) -> str: ...



import getpass
import lzma
import os
import pathlib
import re
import shutil
import sys

import pytest

import py7zr
import py7zr.archiveinfo
import py7zr.callbacks
import py7zr.cli
import py7zr.properties

from . import check_output, decode_all, ltime2

testdata_path = os.path.join(os.path.dirname(__file__), "data")
os.umask(0o022)


@pytest.mark.basic
def test_basic_initinfo():
    archive = py7zr.SevenZipFile(open(os.path.join(testdata_path, "test_1.7z"), "rb"))
    assert archive is not None


@pytest.mark.cli
def test_cli_list_1(capsys):
    arc = os.path.join(testdata_path, "test_1.7z")
    expected = """total 4 files and directories in solid archive
   Date      Time    Attr         Size   Compressed  Name
------------------- ----- ------------ ------------  ------------------------
"""
    expected += "{} D....            0            0  scripts\n".format(ltime2(2019, 3, 14, 0, 10, 8))
    expected += "{} ....A          111          441  scripts/py7zr\n".format(ltime2(2019, 3, 14, 0, 10, 8))
    expected += "{} ....A           58               setup.cfg\n".format(ltime2(2019, 3, 14, 0, 7, 13))
    expected += "{} ....A          559               setup.py\n".format(ltime2(2019, 3, 14, 0, 9, 1))
    expected += "------------------- ----- ------------ ------------  ------------------------\n"
    cli = py7zr.cli.Cli()
    cli.run(["l", arc])
    out, err = capsys.readouterr()
    assert out == expected


@pytest.mark.basic
def test_cli_list_2(capsys):
    arc = os.path.join(testdata_path, "test_3.7z")
    expected = "total 28 files and directories in solid archive\n"
    expected += "   Date      Time    Attr         Size   Compressed  Name\n"
    expected += "------------------- ----- ------------ ------------  ------------------------\n"
    expected += "{} D....            0            0  5.9.7\n".format(ltime2(2018, 10, 18, 14, 52, 42))  # noqa: E501
    expected += "{} D....            0            0  5.9.7/gcc_64\n".format(ltime2(2018, 10, 18, 14, 52, 43))  # noqa: E501
    expected += "{} D....            0            0  5.9.7/gcc_64/include\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} D....            0            0  5.9.7/gcc_64/include/QtX11Extras\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} D....            0            0  5.9.7/gcc_64/lib\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} D....            0            0  5.9.7/gcc_64/lib/cmake\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} D....            0            0  5.9.7/gcc_64/lib/cmake/Qt5X11Extras\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} D....            0            0  5.9.7/gcc_64/lib/pkgconfig\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} D....            0            0  5.9.7/gcc_64/mkspecs\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} D....            0            0  5.9.7/gcc_64/mkspecs/modules\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} ....A           26         8472  5.9.7/gcc_64/include/QtX11Extras/QX11Info\n".format(
        ltime2(2018, 10, 16, 10, 26, 21)
    )  # noqa: E501
    expected += "{} ....A          176               5.9.7/gcc_64/include/QtX11Extras/QtX11Extras\n".format(
        ltime2(2018, 10, 16, 10, 26, 24)
    )  # noqa: E501
    expected += "{} ....A          201               5.9.7/gcc_64/include/QtX11Extras/QtX11ExtrasDepends\n".format(
        ltime2(2018, 10, 16, 10, 26, 24)
    )  # noqa: E501
    expected += "{} ....A           32               5.9.7/gcc_64/include/QtX11Extras/QtX11ExtrasVersion\n".format(
        ltime2(2018, 10, 16, 10, 26, 24)
    )  # noqa: E501
    expected += "{} ....A          722               5.9.7/gcc_64/lib/libQt5X11Extras.la\n".format(
        ltime2(2018, 10, 16, 10, 26, 27)
    )  # noqa: E501
    expected += "{} ....A         2280               5.9.7/gcc_64/include/QtX11Extras/qtx11extrasglobal.h\n".format(
        ltime2(2018, 10, 16, 10, 26, 21)
    )  # noqa: E501
    expected += "{} ....A          222               5.9.7/gcc_64/include/QtX11Extras/qtx11extrasversion.h\n".format(
        ltime2(2018, 10, 16, 10, 26, 24)
    )  # noqa: E501
    expected += "{} ....A         2890               5.9.7/gcc_64/include/QtX11Extras/qx11info_x11.h\n".format(
        ltime2(2018, 10, 16, 10, 26, 21)
    )  # noqa: E501
    expected += "{} ....A           24               5.9.7/gcc_64/lib/libQt5X11Extras.so\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} ....A           24               5.9.7/gcc_64/lib/libQt5X11Extras.so.5\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} ....A        14568               5.9.7/gcc_64/lib/libQt5X11Extras.so.5.9.7\n".format(
        ltime2(2018, 10, 16, 10, 26, 27)
    )  # noqa: E501
    expected += "{} ....A           24               5.9.7/gcc_64/lib/libQt5X11Extras.so.5.9\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} ....A         6704               5.9.7/gcc_64/lib/cmake/Qt5X11Extras/Qt5X11ExtrasConfig.cmake\n".format(
        ltime2(2018, 10, 16, 10, 26, 24)
    )  # noqa: E501
    expected += (
        "{} ....A          287               5.9.7/gcc_64/lib/cmake/Qt5X11Extras/Qt5X11ExtrasConfigVersion.cmake\n".format(
            ltime2(2018, 10, 16, 10, 26, 24)
        )
    )  # noqa: E501
    expected += "{} ....A          283               5.9.7/gcc_64/lib/pkgconfig/Qt5X11Extras.pc\n".format(
        ltime2(2018, 10, 16, 10, 26, 27)
    )  # noqa: E501
    expected += "{} ....A          555               5.9.7/gcc_64/mkspecs/modules/qt_lib_x11extras.pri\n".format(
        ltime2(2018, 10, 16, 10, 26, 24)
    )  # noqa: E501
    expected += "{} ....A          526               5.9.7/gcc_64/mkspecs/modules/qt_lib_x11extras_private.pri\n".format(
        ltime2(2018, 10, 16, 10, 26, 24)
    )  # noqa: E501
    expected += "{} ....A         1064               5.9.7/gcc_64/lib/libQt5X11Extras.prl\n".format(
        ltime2(2018, 10, 18, 10, 28, 16)
    )  # noqa: E501
    expected += "------------------- ----- ------------ ------------  ------------------------\n"
    cli = py7zr.cli.Cli()
    cli.run(["l", arc])
    out, err = capsys.readouterr()
    assert out == expected


@pytest.mark.api
def test_basic_not_implemented_yet1(tmp_path):
    with pytest.raises(NotImplementedError):
        py7zr.SevenZipFile(tmp_path.joinpath("test_x.7z"), mode="x")


@pytest.mark.api
def test_basic_append_mode(tmp_path):
    target = tmp_path.joinpath("test_a.7z")
    shutil.copy(os.path.join(testdata_path, "test_1.7z"), target)
    with py7zr.SevenZipFile(target, mode="a") as archive:
        archive.write(os.path.join(testdata_path, "test1.txt"), "test1.txt")


@pytest.mark.api
def test_basic_wrong_option_value(tmp_path):
    with pytest.raises(ValueError):
        py7zr.SevenZipFile(tmp_path.joinpath("test_p.7z"), mode="p")


@pytest.mark.basic
def test_basic_extract_1(tmp_path):
    archive = py7zr.SevenZipFile(open(os.path.join(testdata_path, "test_1.7z"), "rb"))
    expected = [
        {
            "filename": "setup.cfg",
            "mode": 33188,
            "mtime": 1552522033,
            "digest": "ff77878e070c4ba52732b0c847b5a055a7c454731939c3217db4a7fb4a1e7240",
        },
        {
            "filename": "setup.py",
            "mode": 33188,
            "mtime": 1552522141,
            "digest": "b916eed2a4ee4e48c51a2b51d07d450de0be4dbb83d20e67f6fd166ff7921e49",
        },
        {
            "filename": "scripts/py7zr",
            "mode": 33261,
            "mtime": 1552522208,
            "digest": "b0385e71d6a07eb692f5fb9798e9d33aaf87be7dfff936fd2473eab2a593d4fd",
        },
    ]
    decode_all(archive, expected, tmp_path)


@pytest.mark.basic
def test_basic_extract_2(tmp_path):
    archive = py7zr.SevenZipFile(open(os.path.join(testdata_path, "test_2.7z"), "rb"))
    expected = [
        {
            "filename": "qt.qt5.597.gcc_64/installscript.qs",
            "digest": "39445276e79ea43c0fa8b393b35dc621fcb2045cb82238ddf2b838a4fbf8a587",
        }
    ]
    decode_all(archive, expected, tmp_path)


@pytest.mark.basic
def test_basic_decode_3(tmp_path):
    """Test when passing path string instead of file-like object."""
    archive = py7zr.SevenZipFile(os.path.join(testdata_path, "test_1.7z"))
    expected = [
        {
            "filename": "setup.cfg",
            "mode": 33188,
            "mtime": 1552522033,
            "digest": "ff77878e070c4ba52732b0c847b5a055a7c454731939c3217db4a7fb4a1e7240",
        }
    ]
    decode_all(archive, expected, tmp_path)


@pytest.mark.api
def test_py7zr_is_7zfile():
    assert py7zr.is_7zfile(os.path.join(testdata_path, "test_1.7z"))


@pytest.mark.api
def test_py7zr_is_7zfile_fileish():
    assert py7zr.is_7zfile(open(os.path.join(testdata_path, "test_1.7z"), "rb"))


@pytest.mark.api
def test_py7zr_is_7zfile_path():
    assert py7zr.is_7zfile(pathlib.Path(testdata_path).joinpath("test_1.7z"))


@pytest.mark.basic
def test_py7zr_is_not_7zfile(tmp_path):
    target = tmp_path.joinpath("test_not.7z")
    with target.open("wb") as f:
        f.write(b"12345dahodjg98adfjfak;")
    with target.open("rb") as f:
        assert not py7zr.is_7zfile(f)


@pytest.mark.cli
def test_cli_help(capsys):
    if sys.version_info >= (3, 10):
        expected = "usage: py7zr [-h] [--version] {l,x,c,a,t,i} ...\n\npy7zr\n\noptions:"
    else:
        expected = "usage: py7zr [-h] [--version] {l,x,c,a,t," "i} ...\n\npy7zr\n\noptional arguments:\n  -h, --help"
    cli = py7zr.cli.Cli()
    with pytest.raises(SystemExit):
        cli.run(["-h"])
    out, err = capsys.readouterr()
    assert out.startswith(expected)


@pytest.mark.cli
def test_cli_no_subcommand(capsys):
    expected = py7zr.cli.Cli._get_version()
    if sys.version_info >= (3, 10):
        expected += "\nusage: py7zr [-h] [--version] {l,x,c,a,t,i} ...\n\npy7zr\n\noptions:"
    else:
        expected += "\nusage: py7zr [-h] [--version] {l,x,c,a,t,i} ...\n\npy7zr\n\noptional arguments:\n  -h, --help"
    cli = py7zr.cli.Cli()
    cli.run([])
    out, err = capsys.readouterr()
    assert out.startswith(expected)


@pytest.mark.cli
def test_cli_list_verbose(capsys):
    arcfile = os.path.join(testdata_path, "test_1.7z")
    expected = """Listing archive: {}
--
Path = {}
Type = 7z
Phisical Size = 657
Headers Size = 216
Method = LZMA2
Solid = +
Blocks = 1

total 4 files and directories in solid archive
   Date      Time    Attr         Size   Compressed  Name
------------------- ----- ------------ ------------  ------------------------
""".format(
        arcfile, arcfile
    )
    expected += "{} D....            0            0  scripts\n".format(ltime2(2019, 3, 14, 0, 10, 8))
    expected += "{} ....A          111          441  scripts/py7zr\n".format(ltime2(2019, 3, 14, 0, 10, 8))
    expected += "{} ....A           58               setup.cfg\n".format(ltime2(2019, 3, 14, 0, 7, 13))
    expected += "{} ....A          559               setup.py\n".format(ltime2(2019, 3, 14, 0, 9, 1))
    expected += "------------------- ----- ------------ ------------  ------------------------\n"
    cli = py7zr.cli.Cli()
    cli.run(["l", "--verbose", arcfile])
    out, err = capsys.readouterr()
    assert out == expected


@pytest.mark.cli
def test_cli_test(capsys):
    arcfile = os.path.join(testdata_path, "test_2.7z")
    expected = """Testing archive: {}
--
Path = {}
Type = 7z
Phisical Size = 1663
Headers Size = 203
Method = LZMA2
Solid = -
Blocks = 1

Everything is Ok
""".format(
        arcfile, arcfile
    )
    cli = py7zr.cli.Cli()
    cli.run(["t", arcfile])
    out, err = capsys.readouterr()
    assert out == expected


@pytest.mark.cli
def test_cli_info(capsys):
    expected = py7zr.cli.Cli._get_version()
    if lzma.is_check_supported(lzma.CHECK_CRC64):
        check0 = "\n0              CRC64"
    else:
        check0 = ""
    if lzma.is_check_supported(lzma.CHECK_SHA256):
        check1 = "\n0             SHA256"
    else:
        check1 = ""
    expected_checks = """0              CRC32{}{}""".format(check1, check0)
    expected += """\n
Formats:
7z    37 7a bc af 27 1c

Codecs and hashes:
00              COPY
21             LZMA2
03             DELTA
030101          LZMA
03030103         BCJ
03030205         PPC
03030401        IA64
03030501         ARM
03030701        ARMT
03030805       SPARC
040108       DEFLATE
040202         BZip2
04f71101   ZStandard
030401          PPMd
04f71102      Brotli
06f10701       7zAES
{}
""".format(
        expected_checks
    )
    cli = py7zr.cli.Cli()
    cli.run(["i"])
    out, err = capsys.readouterr()
    assert out == expected


@pytest.mark.cli
def test_cli_extract(tmp_path):
    arcfile = os.path.join(testdata_path, "test_1.7z")
    cli = py7zr.cli.Cli()
    cli.run(["x", arcfile, str(tmp_path.resolve())])
    expected = [
        {
            "filename": "setup.cfg",
            "mode": 33188,
            "mtime": 1552522033,
            "digest": "ff77878e070c4ba52732b0c847b5a055a7c454731939c3217db4a7fb4a1e7240",
        },
        {
            "filename": "setup.py",
            "mode": 33188,
            "mtime": 1552522141,
            "digest": "b916eed2a4ee4e48c51a2b51d07d450de0be4dbb83d20e67f6fd166ff7921e49",
        },
        {
            "filename": "scripts/py7zr",
            "mode": 33261,
            "mtime": 1552522208,
            "digest": "b0385e71d6a07eb692f5fb9798e9d33aaf87be7dfff936fd2473eab2a593d4fd",
        },
    ]
    check_output(expected, tmp_path)


@pytest.mark.cli
def test_cli_encrypted_extract(monkeypatch, tmp_path):
    def _getpasswd():
        return "secret"

    monkeypatch.setattr(getpass, "getpass", _getpasswd)

    arcfile = os.path.join(testdata_path, "encrypted_1.7z")
    cli = py7zr.cli.Cli()
    cli.run(["x", "--password", arcfile, str(tmp_path.resolve())])
    expected = [
        {
            "filename": "test1.txt",
            "mode": 33188,
            "digest": "0f16b2f4c3a74b9257cd6229c0b7b91855b3260327ef0a42ecf59c44d065c5b2",
        },
        {
            "filename": "test/test2.txt",
            "mode": 33188,
            "digest": "1d0d28682fca74c5912ea7e3f6878ccfdb6e4e249b161994b7f2870e6649ef09",
        },
    ]
    check_output(expected, tmp_path)


@pytest.mark.cli
def test_cli_encrypted_wrong_password(monkeypatch, tmp_path, capsys):
    def _getpasswd():
        return "badsecret"

    monkeypatch.setattr(getpass, "getpass", _getpasswd)

    expected = "The archive is corrupted, or password is wrong. ABORT.\n"

    arcfile = os.path.join(testdata_path, "encrypted_1.7z")
    cli = py7zr.cli.Cli()
    result = cli.run(["x", "--password", arcfile, str(tmp_path.resolve())])
    out, err = capsys.readouterr()
    assert result == 1
    assert out == expected


@pytest.mark.cli
@pytest.mark.skipif(sys.hexversion == 0x030900F0, reason="bpo-42057")
def test_cli_encrypted_zero_length_password(monkeypatch, tmp_path, capsys):
    def _getpasswd():
        return ""

    monkeypatch.setattr(getpass, "getpass", _getpasswd)

    expected = "The archive is corrupted, or password is wrong. ABORT.\n"

    arcfile = os.path.join(testdata_path, "encrypted_1.7z")
    cli = py7zr.cli.Cli()
    result = cli.run(["x", "--password", arcfile, str(tmp_path.resolve())])
    out, err = capsys.readouterr()
    assert result == 1
    assert out == expected


@pytest.mark.basic
def test_digests():
    arcfile = os.path.join(testdata_path, "test_2.7z")
    archive = py7zr.SevenZipFile(arcfile)
    assert archive.test() is None
    assert archive.testzip() is None


@pytest.mark.basic
def test_digests_corrupted():
    arcfile = os.path.join(testdata_path, "crc_corrupted.7z")
    with py7zr.SevenZipFile(arcfile) as archive:
        assert archive.test() is None
        archive.reset()
        assert archive.testzip() == "src/scripts/py7zr"


@pytest.mark.cli
def test_non7z_ext(capsys, tmp_path):
    expected = "not a 7z file\n"
    arcfile = os.path.join(testdata_path, "test_1.txt")
    cli = py7zr.cli.Cli()
    cli.run(["x", arcfile, str(tmp_path.resolve())])
    out, err = capsys.readouterr()
    assert out == expected


@pytest.mark.cli
def test_non7z_test(capsys):
    expected = "not a 7z file\n"
    arcfile = os.path.join(testdata_path, "test_1.txt")
    cli = py7zr.cli.Cli()
    cli.run(["t", arcfile])
    out, err = capsys.readouterr()
    assert out == expected


@pytest.mark.cli
def test_non7z_list(capsys):
    expected = "not a 7z file\n"
    arcfile = os.path.join(testdata_path, "test_1.txt")
    cli = py7zr.cli.Cli()
    cli.run(["l", arcfile])
    out, err = capsys.readouterr()
    assert out == expected


@pytest.mark.cli
def test_archive_creation(tmp_path, capsys):
    tmp_path.joinpath("src").mkdir()
    py7zr.unpack_7zarchive(os.path.join(testdata_path, "test_1.7z"), path=tmp_path.joinpath("src"))
    os.chdir(str(tmp_path))
    target = "target.7z"
    source = "src"
    cli = py7zr.cli.Cli()
    cli.run(["c", target, source])
    out, err = capsys.readouterr()


@pytest.mark.cli
@pytest.mark.skipif(sys.version_info < (3, 7), reason="requires python3.7 or higher")
def test_archive_already_exist(tmp_path, capsys):
    expected = "Archive file exists!\n"
    py7zr.unpack_7zarchive(os.path.join(testdata_path, "test_1.7z"), path=tmp_path.joinpath("src"))
    target = tmp_path / "target.7z"
    with target.open("w") as f:
        f.write("Already exist!")
    source = str(tmp_path / "src")
    cli = py7zr.cli.Cli()
    with pytest.raises(SystemExit):
        cli.run(["c", str(target), source])
    out, err = capsys.readouterr()
    assert err == expected


@pytest.mark.cli
def test_archive_append(tmp_path):
    py7zr.unpack_7zarchive(os.path.join(testdata_path, "test_2.7z"), path=tmp_path.joinpath("src"))
    target = tmp_path / "target.7z"
    shutil.copy(os.path.join(testdata_path, "test_1.7z"), target)
    source = str(tmp_path / "src")
    cli = py7zr.cli.Cli()
    cli.run(["a", str(target), source])


@pytest.mark.cli
def test_archive_without_extension(tmp_path, capsys):
    py7zr.unpack_7zarchive(os.path.join(testdata_path, "test_1.7z"), path=tmp_path.joinpath("src"))
    target = str(tmp_path / "target")
    source = str(tmp_path / "src")
    cli = py7zr.cli.Cli()
    cli.run(["c", target, source])
    expected_target = tmp_path / "target.7z"
    assert expected_target.exists()


@pytest.mark.cli
@pytest.mark.skipif(sys.version_info < (3, 7), reason="requires python3.7 or higher")
def test_volume_creation(tmp_path, capsys):
    tmp_path.joinpath("src").mkdir()
    py7zr.unpack_7zarchive(os.path.join(testdata_path, "lzma2bcj.7z"), path=tmp_path.joinpath("src"))
    target = str(tmp_path / "target.7z")
    source = str(tmp_path / "src")
    cli = py7zr.cli.Cli()
    cli.run(["c", target, source, "-v", "2m"])
    out, err = capsys.readouterr()


@pytest.mark.cli
@pytest.mark.skipif(sys.version_info < (3, 7), reason="requires python3.7 or higher")
def test_volume_creation_wrong_volume_unit(tmp_path, capsys):
    expected = "Error: Specified volume size is invalid.\n"
    target = str(tmp_path / "target.7z")
    source = tmp_path / "src"
    source.mkdir()
    cli = py7zr.cli.Cli()
    with pytest.raises(SystemExit):
        cli.run(["c", target, str(source), "-v", "2P"])
    out, err = capsys.readouterr()
    assert err == expected


@pytest.mark.unit
def test_py7zr_write_mode(tmp_path):
    target = tmp_path.joinpath("target.7z")
    archive = py7zr.SevenZipFile(target, "w")
    archive.write(os.path.join(testdata_path, "test1.txt"), "test1.txt")
    assert archive.files is not None
    assert len(archive.files) == 1
    for f in archive.files:
        assert f.filename in ("test1.txt")
        assert not f.emptystream


@pytest.mark.api
def test_py7zr_writeall_single(tmp_path):
    target = tmp_path.joinpath("target.7z")
    archive = py7zr.SevenZipFile(target, "w")
    archive.writeall(os.path.join(testdata_path, "test1.txt"), "test1.txt")
    assert archive.files is not None
    assert len(archive.files) == 1
    for f in archive.files:
        assert f.filename in ("test1.txt")
        assert not f.emptystream


@pytest.mark.api
def test_py7zr_writeall_dir(tmp_path):
    target = tmp_path.joinpath("target.7z")
    archive = py7zr.SevenZipFile(target, "w")
    archive.writeall(os.path.join(testdata_path, "src"), "src")
    assert archive.files is not None
    assert len(archive.files) == 2
    for f in archive.files:
        assert f.filename in ("src", "src/bra.txt")
    archive._fpclose()


@pytest.mark.api
def test_py7zr_extract_specified_file(tmp_path):
    archive = py7zr.SevenZipFile(open(os.path.join(testdata_path, "test_1.7z"), "rb"))
    expected = [
        {
            "filename": "scripts/py7zr",
            "mode": 33261,
            "mtime": 1552522208,
            "digest": "b0385e71d6a07eb692f5fb9798e9d33aaf87be7dfff936fd2473eab2a593d4fd",
        }
    ]
    archive.extract(path=tmp_path, targets=["scripts", "scripts/py7zr"])
    archive.close()
    assert tmp_path.joinpath("scripts").is_dir()
    assert tmp_path.joinpath("scripts/py7zr").exists()
    assert not tmp_path.joinpath("setup.cfg").exists()
    assert not tmp_path.joinpath("setup.py").exists()
    check_output(expected, tmp_path)


@pytest.mark.api
def test_py7zr_extract_and_getnames(tmp_path):
    archive = py7zr.SevenZipFile(open(os.path.join(testdata_path, "test_1.7z"), "rb"))
    allfiles = archive.getnames()
    filter_pattern = re.compile(r"scripts.*")
    targets = []
    for f in allfiles:
        if filter_pattern.match(f):
            targets.append(f)
    archive.extract(path=tmp_path, targets=targets)
    archive.close()
    assert tmp_path.joinpath("scripts").is_dir()
    assert tmp_path.joinpath("scripts/py7zr").exists()
    assert not tmp_path.joinpath("setup.cfg").exists()
    assert not tmp_path.joinpath("setup.py").exists()


@pytest.mark.api
def test_py7zr_extract_and_reset_iteration(tmp_path):
    archive = py7zr.SevenZipFile(open(os.path.join(testdata_path, "test_1.7z"), "rb"))
    iterations = archive.getnames()
    for target in iterations:
        archive.extract(path=tmp_path, targets=[target])
        archive.reset()
    archive.close()
    assert tmp_path.joinpath("scripts").is_dir()
    assert tmp_path.joinpath("scripts/py7zr").exists()
    assert tmp_path.joinpath("setup.cfg").exists()
    assert tmp_path.joinpath("setup.py").exists()


@pytest.mark.api
def test_context_manager_1(tmp_path):
    with py7zr.SevenZipFile(os.path.join(testdata_path, "test_1.7z"), "r") as z:
        z.extractall(path=tmp_path)
    assert tmp_path.joinpath("scripts").is_dir()
    assert tmp_path.joinpath("scripts/py7zr").exists()
    assert tmp_path.joinpath("setup.cfg").exists()
    assert tmp_path.joinpath("setup.py").exists()


@pytest.mark.api
def test_context_manager_2(tmp_path):
    target = tmp_path.joinpath("target.7z")
    with py7zr.SevenZipFile(target, "w") as z:
        z.writeall(os.path.join(testdata_path, "src"), "src")


@pytest.mark.api
def test_extract_callback(tmp_path):
    class ECB(py7zr.callbacks.ExtractCallback):
        def __init__(self, ofd):
            self.ofd = ofd

        def report_start_preparation(self):
            self.ofd.write("preparation.\n")

        def report_start(self, processing_file_path, processing_bytes):
            self.ofd.write('start "{}" (compressed in {} bytes)\n'.format(processing_file_path, processing_bytes))

        def report_end(self, processing_file_path, wrote_bytes):
            self.ofd.write('end "{}" extracted to {} bytes\n'.format(processing_file_path, wrote_bytes))

        def report_postprocess(self):
            self.ofd.write("post processing.\n")

        def report_warning(self, message):
            self.ofd.write("warning: {:s}\n".format(message))

    cb = ECB(sys.stdout)
    with py7zr.SevenZipFile(open(os.path.join(testdata_path, "test_1.7z"), "rb")) as archive:
        archive.extractall(path=tmp_path, callback=cb)


@pytest.mark.api
def test_py7zr_list_values():
    with py7zr.SevenZipFile(os.path.join(testdata_path, "test_1.7z"), "r") as z:
        file_list = z.list()
    assert file_list[0].filename == "scripts"
    assert file_list[1].filename == "scripts/py7zr"
    assert file_list[2].filename == "setup.cfg"
    assert file_list[3].filename == "setup.py"
    assert file_list[0].uncompressed == 0
    assert file_list[1].uncompressed == 111
    assert file_list[2].uncompressed == 58
    assert file_list[3].uncompressed == 559
    assert file_list[0].is_directory is True
    assert file_list[1].archivable is True
    assert file_list[2].archivable is True
    assert file_list[3].archivable is True
    assert file_list[0].compressed == 0
    assert file_list[1].compressed == 441
    assert file_list[2].compressed is None
    assert file_list[3].compressed is None
    assert file_list[0].crc32 is None
    assert file_list[1].crc32 == 0xB36AAEDB
    assert file_list[2].crc32 == 0xDCBF8D07
    assert file_list[3].crc32 == 0x80FC72BE


@pytest.mark.cli
@pytest.mark.files
def test_list_multivolume(capsys):
    arcfile = os.path.join(testdata_path, "archive.7z.001")
    basefile = os.path.join(testdata_path, "archive.7z")
    expected = """Listing archive: {}
--
Path = {}
Type = 7z
Phisical Size = 52337
Headers Size = 518
Method = LZMA2, BCJ
Solid = +
Blocks = 2

total 19 files and directories in solid archive
   Date      Time    Attr         Size   Compressed  Name
------------------- ----- ------------ ------------  ------------------------
""".format(
        basefile, basefile
    )
    expected += "{} D....            0            0  mingw64\n".format(ltime2(2017, 1, 23, 6, 2, 46))
    expected += "{} D....            0            0  mingw64/bin\n".format(ltime2(2020, 6, 7, 2, 45, 18))
    expected += "{} D....            0            0  mingw64/include\n".format(ltime2(2020, 6, 7, 2, 45, 18))
    expected += "{} D....            0            0  mingw64/lib\n".format(ltime2(2020, 6, 7, 2, 45, 18))
    expected += "{} D....            0            0  mingw64/share\n".format(ltime2(2020, 6, 7, 2, 45, 26))
    expected += "{} D....            0            0  mingw64/share/doc\n".format(ltime2(2017, 1, 23, 6, 2, 43))
    expected += "{} D....            0            0  mingw64/share/doc/szip\n".format(ltime2(2017, 1, 23, 6, 2, 43))
    expected += "{} ....A         2289        26895  mingw64/include/SZconfig.h\n".format(ltime2(2017, 1, 23, 6, 2, 34))
    expected += "{} ....A         3470               mingw64/include/ricehdf.h\n".format(ltime2(2004, 3, 16, 16, 14, 27))
    expected += "{} ....A         1774               mingw64/include/szip_adpt.h\n".format(ltime2(2010, 7, 2, 21, 31, 38))
    expected += "{} ....A         5282               mingw64/include/szlib.h\n".format(ltime2(2008, 11, 11, 16, 12, 56))
    expected += "{} ....A        60008               mingw64/lib/libszip.a\n".format(ltime2(2017, 1, 23, 6, 2, 47))
    expected += "{} ....A        10900               mingw64/lib/libszip.dll.a\n".format(ltime2(2017, 1, 23, 6, 2, 39))
    expected += "{} ....A         1986               mingw64/share/doc/szip/COPYING\n".format(ltime2(2008, 1, 24, 23, 8, 43))
    expected += "{} ....A         1544               mingw64/share/doc/szip/HISTORY.txt\n".format(
        ltime2(2010, 7, 14, 13, 43, 15)
    )
    expected += "{} ....A         3544               mingw64/share/doc/szip/INSTALL\n".format(
        ltime2(2008, 11, 11, 16, 12, 56)
    )
    expected += "{} ....A          564               mingw64/share/doc/szip/README\n".format(ltime2(2007, 8, 20, 18, 47, 21))
    expected += "{} ....A          513               mingw64/share/doc/szip/RELEASE.txt\n".format(
        ltime2(2010, 7, 14, 13, 43, 15)
    )
    expected += "{} ....A        66352        24924  mingw64/bin/libszip-0.dll\n".format(ltime2(2017, 1, 23, 6, 2, 47))
    expected += "------------------- ----- ------------ ------------  ------------------------\n"
    cli = py7zr.cli.Cli()
    cli.run(["l", "--verbose", arcfile])
    out, err = capsys.readouterr()
    assert out == expected



from io import BytesIO
from py7zr.compressor import SevenZipCompressor as SevenZipCompressor, SevenZipDecompressor as SevenZipDecompressor
from py7zr.exceptions import Bad7zFile as Bad7zFile
from py7zr.helpers import ArchiveTimestamp as ArchiveTimestamp, calculate_crc32 as calculate_crc32
from py7zr.properties import DEFAULT_FILTERS as DEFAULT_FILTERS, MAGIC_7Z as MAGIC_7Z, PROPERTY as PROPERTY
from typing import Any, BinaryIO, List, Tuple

MAX_LENGTH: int
P7ZIP_MAJOR_VERSION: bytes
P7ZIP_MINOR_VERSION: bytes

def read_crcs(file: BinaryIO, count: int) -> List[int]: ...
def write_crcs(file: BinaryIO, crcs: Any) -> Any: ...
def read_byte(file: BinaryIO) -> int: ...
def write_bytes(file: BinaryIO, data: bytes) -> Any: ...
def write_byte(file: BinaryIO, data: Any) -> Any: ...
def read_real_uint64(file: BinaryIO) -> Tuple[int, bytes]: ...
def read_uint32(file: BinaryIO) -> Tuple[int, bytes]: ...
def write_uint32(file: BinaryIO, value: Any) -> Any: ...
def read_uint64(file: BinaryIO) -> int: ...
def write_real_uint64(file: BinaryIO, value: int) -> Any: ...
def write_uint64(file: BinaryIO, value: int) -> Any: ...
def read_boolean(file: BinaryIO, count: int, checkall: bool=...) -> List[bool]: ...
def write_boolean(file: BinaryIO, booleans: List[bool], all_defined: bool=...) -> Any: ...
def read_utf16(file: BinaryIO) -> str: ...
def write_utf16(file: BinaryIO, val: str) -> Any: ...
def bits_to_bytes(bit_length: int) -> int: ...

class PackInfo:
    packpos: int = ...
    numstreams: int = ...
    packsizes: Any = ...
    digestdefined: Any = ...
    crcs: Any = ...
    enable_digests: bool = ...
    def __init__(self) -> None: ...
    @classmethod
    def retrieve(cls, file: BinaryIO) -> Any: ...
    def write(self, file: BinaryIO) -> Any: ...

class Bond:
    incoder: Any = ...
    outcoder: Any = ...
    def __init__(self, incoder: Any, outcoder: Any) -> None: ...

class Folder:
    unpacksizes: Any = ...
    coders: Any = ...
    bindpairs: Any = ...
    packed_indices: Any = ...
    solid: bool = ...
    digestdefined: bool = ...
    crc: Any = ...
    decompressor: Any = ...
    compressor: Any = ...
    files: Any = ...
    password: Any = ...
    def __init__(self) -> None: ...
    @classmethod
    def retrieve(cls, file: BinaryIO) -> Any: ...
    def prepare_coderinfo(self, filters: Any) -> None: ...
    def write(self, file: BinaryIO) -> Any: ...
    def is_simple(self, coder: Any): ...
    def get_decompressor(self, packsize: int, reset: bool=...) -> SevenZipDecompressor: ...
    def get_compressor(self) -> SevenZipCompressor: ...
    def get_unpack_size(self) -> int: ...

class UnpackInfo:
    @classmethod
    def retrieve(cls, file: BinaryIO) -> Any: ...
    numfolders: Any = ...
    folders: Any = ...
    datastreamidx: Any = ...
    def __init__(self) -> None: ...
    def write(self, file: BinaryIO) -> Any: ...

class SubstreamsInfo:
    digests: Any = ...
    digestsdefined: Any = ...
    unpacksizes: Any = ...
    num_unpackstreams_folders: Any = ...
    def __init__(self) -> None: ...
    @classmethod
    def retrieve(cls, file: BinaryIO, numfolders: int, folders: List[Folder]) -> Any: ...
    def write(self, file: BinaryIO) -> Any: ...

class StreamsInfo:
    packinfo: Any = ...
    unpackinfo: Any = ...
    substreamsinfo: Any = ...
    def __init__(self) -> None: ...
    @classmethod
    def retrieve(cls, file: BinaryIO) -> Any: ...
    def read(self, file: BinaryIO) -> None: ...
    def write(self, file: BinaryIO) -> Any: ...

class HeaderStreamsInfo(StreamsInfo):
    packinfo: Any = ...
    unpackinfo: Any = ...
    def __init__(self) -> None: ...
    def write(self, file: BinaryIO) -> Any: ...

class FilesInfo:
    files: Any = ...
    emptyfiles: Any = ...
    antifiles: Any = ...
    def __init__(self) -> None: ...
    @classmethod
    def retrieve(cls, file: BinaryIO) -> Any: ...
    def write(self, file: BinaryIO) -> Any: ...

class WriteWithCrc:
    digest: int = ...
    def __init__(self, fp: BinaryIO) -> None: ...
    def write(self, data: Any): ...
    def tell(self): ...

class Header:
    __slot__: Any = ...
    solid: bool = ...
    main_streams: Any = ...
    files_info: Any = ...
    size: int = ...
    password: Any = ...
    def __init__(self) -> None: ...
    @classmethod
    def retrieve(cls, fp: BinaryIO, buffer: BytesIO, start_pos: int, password: Any=...) -> Any: ...
    def write(self, file: BinaryIO, afterheader: int, encoded: Any=..., encrypted: Any=...) -> Any: ...
    @staticmethod
    def build_header(folders: Any): ...

class SignatureHeader:
    version: Any = ...
    startheadercrc: int = ...
    nextheaderofs: int = ...
    nextheadersize: int = ...
    nextheadercrc: int = ...
    def __init__(self) -> None: ...
    @classmethod
    def retrieve(cls, file: BinaryIO) -> Any: ...
    def calccrc(self, length: int, header_crc: int) -> Any: ...
    def write(self, file: BinaryIO) -> Any: ...



import abc
from abc import ABC, abstractmethod
from typing import Any

class Callback(ABC, metaclass=abc.ABCMeta):
    @abstractmethod
    def report_start_preparation(self) -> Any: ...
    @abstractmethod
    def report_start(self, processing_file_path: Any, processing_bytes: Any) -> Any: ...
    @abstractmethod
    def report_end(self, processing_file_path: Any, wrote_bytes: Any) -> Any: ...
    @abstractmethod
    def report_warning(self, message: Any) -> Any: ...
    @abstractmethod
    def report_postprocess(self) -> Any: ...

class ExtractCallback(Callback, metaclass=abc.ABCMeta): ...
class ArchiveCallback(Callback, metaclass=abc.ABCMeta): ...



import collections.abc
import contextlib
import pathlib
import queue
from py7zr.archiveinfo import Folder as Folder, Header as Header, SignatureHeader as SignatureHeader
from py7zr.callbacks import ExtractCallback as ExtractCallback
from py7zr.compressor import SupportedMethods as SupportedMethods, get_methods_names_string as get_methods_names_string
from py7zr.exceptions import Bad7zFile as Bad7zFile, CrcError as CrcError, DecompressionError as DecompressionError, InternalError as InternalError, UnsupportedCompressionMethodError as UnsupportedCompressionMethodError
from py7zr.helpers import ArchiveTimestamp as ArchiveTimestamp, MemIO as MemIO, NullIO as NullIO, calculate_crc32 as calculate_crc32, filetime_to_dt as filetime_to_dt, readlink as readlink
from py7zr.properties import DEFAULT_FILTERS as DEFAULT_FILTERS, MAGIC_7Z as MAGIC_7Z, get_default_blocksize as get_default_blocksize
from typing import Any, BinaryIO, Dict, IO, List, Optional, Union

FILE_ATTRIBUTE_UNIX_EXTENSION: int
FILE_ATTRIBUTE_WINDOWS_MASK: int

class ArchiveFile:
    id: Any = ...
    def __init__(self, id: int, file_info: Dict[str, Any]) -> None: ...
    def file_properties(self) -> Dict[str, Any]: ...
    @property
    def origin(self) -> pathlib.Path: ...
    @property
    def folder(self) -> Folder: ...
    @property
    def filename(self) -> str: ...
    @property
    def emptystream(self) -> bool: ...
    @property
    def uncompressed(self) -> List[int]: ...
    @property
    def compressed(self) -> Optional[int]: ...
    @property
    def crc32(self) -> Optional[int]: ...
    @property
    def archivable(self) -> bool: ...
    @property
    def is_directory(self) -> bool: ...
    @property
    def readonly(self) -> bool: ...
    def data(self) -> Optional[BinaryIO]: ...
    def has_strdata(self) -> bool: ...
    @property
    def is_symlink(self) -> bool: ...
    @property
    def is_junction(self) -> bool: ...
    @property
    def is_socket(self) -> bool: ...
    @property
    def lastwritetime(self) -> Optional[ArchiveTimestamp]: ...
    @property
    def posix_mode(self) -> Optional[int]: ...
    @property
    def st_fmt(self) -> Optional[int]: ...

class ArchiveFileList(collections.abc.Iterable):
    files_list: Any = ...
    index: int = ...
    offset: Any = ...
    def __init__(self, offset: int=...) -> None: ...
    def append(self, file_info: Dict[str, Any]) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> ArchiveFileListIterator: ...
    def __getitem__(self, index: Any): ...

class ArchiveFileListIterator(collections.abc.Iterator):
    def __init__(self, archive_file_list: Any) -> None: ...
    def __next__(self) -> ArchiveFile: ...

class ArchiveInfo:
    stat: Any = ...
    filename: Any = ...
    size: Any = ...
    header_size: Any = ...
    method_names: Any = ...
    solid: Any = ...
    blocks: Any = ...
    uncompressed: Any = ...
    def __init__(self, filename: Any, stat: Any, header_size: Any, method_names: Any, solid: Any, blocks: Any, uncompressed: Any) -> None: ...

class FileInfo:
    filename: Any = ...
    compressed: Any = ...
    uncompressed: Any = ...
    archivable: Any = ...
    is_directory: Any = ...
    creationtime: Any = ...
    crc32: Any = ...
    def __init__(self, filename: Any, compressed: Any, uncompressed: Any, archivable: Any, is_directory: Any, creationtime: Any, crc32: Any) -> None: ...

class SevenZipFile(contextlib.AbstractContextManager):
    mp: Any = ...
    password_protected: Any = ...
    filename: Any = ...
    fp: Any = ...
    mode: Any = ...
    encoded_header_mode: bool = ...
    header_encryption: Any = ...
    worker: Any = ...
    dereference: Any = ...
    reporterd: Any = ...
    q: Any = ...
    def __init__(self, file: Union[BinaryIO, str, pathlib.Path], mode: str=..., *, filters: Optional[List[Dict[str, int]]]=..., dereference: Any=..., password: Optional[str]=..., header_encryption: bool=..., blocksize: Optional[int]=..., mp: bool=...) -> None: ...
    def __enter__(self): ...
    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None: ...
    class ParseStatus:
        src_pos: Any = ...
        folder: int = ...
        outstreams: int = ...
        input: int = ...
        stream: int = ...
        def __init__(self, src_pos: int = ...) -> None: ...
    def set_encoded_header_mode(self, mode: bool) -> None: ...
    def set_encrypted_header(self, mode: bool) -> None: ...
    def getnames(self) -> List[str]: ...
    def archiveinfo(self) -> ArchiveInfo: ...
    def needs_password(self) -> bool: ...
    def list(self) -> List[FileInfo]: ...
    def readall(self) -> Optional[Dict[str, IO[Any]]]: ...
    def extractall(self, path: Optional[Any]=..., callback: Optional[ExtractCallback]=...) -> None: ...
    def read(self, targets: Optional[List[str]]=...) -> Optional[Dict[str, IO[Any]]]: ...
    def extract(self, path: Optional[Any]=..., targets: Optional[List[str]]=...) -> None: ...
    def reporter(self, callback: ExtractCallback) -> Any: ...
    def writeall(self, path: Union[pathlib.Path, str], arcname: Optional[str]=...) -> Any: ...
    def write(self, file: Union[pathlib.Path, str], arcname: Optional[str]=...) -> Any: ...
    def writed(self, targets: Dict[str, IO[Any]]) -> None: ...
    def writef(self, bio: IO[Any], arcname: str) -> Any: ...
    def writestr(self, data: Union[str, bytes, bytearray, memoryview], arcname: str) -> Any: ...
    def close(self) -> None: ...
    def reset(self) -> None: ...
    def test(self) -> Optional[bool]: ...
    def testzip(self) -> Optional[str]: ...

def is_7zfile(file: Union[BinaryIO, str, pathlib.Path]) -> bool: ...
def unpack_7zarchive(archive: Any, path: Any, extra: Optional[Any] = ...) -> None: ...
def pack_7zarchive(base_name: Any, base_dir: Any, owner: Optional[Any] = ..., group: Optional[Any] = ..., dry_run: Optional[Any] = ..., logger: Optional[Any] = ...) -> None: ...

class Worker:
    target_filepath: Any = ...
    files: Any = ...
    src_start: Any = ...
    header: Any = ...
    current_file_index: Any = ...
    last_file_index: Any = ...
    concurrent: Any = ...
    def __init__(self, files: Any, src_start: int, header: Any, mp: Any=...) -> None: ...
    def extract(self, fp: BinaryIO, parallel: bool, skip_notarget: Any=..., q: Any=...) -> None: ...
    def extract_single(self, fp: Union[BinaryIO, str], files: Any, src_start: int, src_end: int, q: Optional[queue.Queue], exc_q: Optional[queue.Queue]=..., skip_notarget: Any=...) -> None: ...
    def decompress(self, fp: BinaryIO, folder: Any, fq: IO[Any], size: int, compressed_size: Optional[int], src_end: int) -> int: ...
    def write(self, fp: BinaryIO, f: Any, assym: Any, folder: Any) -> Any: ...
    def writestr(self, fp: BinaryIO, f: Any, folder: Any) -> Any: ...
    def prepare_archive(self) -> None: ...
    def flush_archive(self, fp: Any, folder: Any) -> None: ...
    def archive(self, fp: BinaryIO, files: Any, folder: Any, deref: Any=...) -> Any: ...
    def register_filelike(self, id: int, fileish: Union[MemIO, pathlib.Path, None]) -> None: ...



from py7zr.exceptions import Bad7zFile as Bad7zFile, DecompressionError as DecompressionError, PasswordRequired as PasswordRequired, UnsupportedCompressionMethodError as UnsupportedCompressionMethodError
from py7zr.properties import CHECK_CRC32 as CHECK_CRC32, CHECK_CRC64 as CHECK_CRC64, CHECK_NONE as CHECK_NONE, CHECK_SHA256 as CHECK_SHA256, FILTER_ARM as FILTER_ARM, FILTER_ARMTHUMB as FILTER_ARMTHUMB, FILTER_BROTLI as FILTER_BROTLI, FILTER_BZIP2 as FILTER_BZIP2, FILTER_COPY as FILTER_COPY, FILTER_CRYPTO_AES256_SHA256 as FILTER_CRYPTO_AES256_SHA256, FILTER_DEFLATE as FILTER_DEFLATE, FILTER_DELTA as FILTER_DELTA, FILTER_IA64 as FILTER_IA64, FILTER_LZMA as FILTER_LZMA, FILTER_LZMA2 as FILTER_LZMA2, FILTER_POWERPC as FILTER_POWERPC, FILTER_PPMD as FILTER_PPMD, FILTER_SPARC as FILTER_SPARC, FILTER_X86 as FILTER_X86, FILTER_ZSTD as FILTER_ZSTD, PRESET_DEFAULT as PRESET_DEFAULT, PRESET_EXTREME as PRESET_EXTREME
from py7zr.py7zr import ArchiveInfo as ArchiveInfo, FileInfo as FileInfo, SevenZipFile as SevenZipFile, is_7zfile as is_7zfile, pack_7zarchive as pack_7zarchive, unpack_7zarchive as unpack_7zarchive

# Names in __all__ with no definition:
#   __version__
